@@ -23,7 +23,6 @@
 * the "all-in-one" configuration to get a web instance running in a twisted
   web server integrating a repository server in the same process (only available
   if the repository part of the software is installed
-
 """
 __docformat__ = "restructuredtext en"
 
@@ -31,8 +30,10 @@ from os.path import join
 
 from logilab.common.configuration import Method
 
+from cubicweb.cwconfig import CONFIGURATIONS
 from cubicweb.web.webconfig import WebConfiguration, merge_options
 
+
 class TwistedConfiguration(WebConfiguration):
     """web instance (in a twisted web server) client of a RQL server"""
     name = 'twisted'
@@ -49,7 +50,7 @@ class TwistedConfiguration(WebConfiguration):
          {'type' : 'string',
           'default': "",
           'help': 'http server address on which to listen (default to everywhere)',
-          'group': 'web', 'level': 0,
+          'group': 'web', 'level': 1,
           }),
         ('max-post-length',
          {'type' : 'bytes',
@@ -98,6 +99,9 @@ the repository rather than the user running the command',
         from socket import gethostname
         return 'http://%s:%s/' % (self['host'] or gethostname(), self['port'] or 8080)
 
+
+CONFIGURATIONS.append(TwistedConfiguration)
+
 try:
     from cubicweb.server.serverconfig import ServerConfiguration
 
@@ -114,5 +118,8 @@ try:
             """tell if pyro is activated for the in memory repository"""
             return self['pyro-server']
 
+
+    CONFIGURATIONS.append(AllInOneConfiguration)
+
 except ImportError:
     pass
