@@ -492,8 +492,6 @@ class ConfigurationProblem(object):
                         self.warnings.append(
                             'cube %s depends on %s but constraint badly '
                             'formatted: %s' % (cube, name, constraint))
-                else:
-                    self.reverse_dependencies[name].add( (None, None, cube) )
         # check consistency
         for cube, versions in sorted(self.reverse_dependencies.items()):
             oper, version, source = None, None, None
@@ -519,7 +517,5 @@ class ConfigurationProblem(object):
                 if oper in ('>=','='):
                     if lower_strict:
                         self.errors.append( ('update', cube, version, source) )
-                elif oper is None:
-                    pass # no constraint on version
                 else:
                     print 'unknown operator', oper
