@@ -544,10 +544,11 @@ class CubicWebSchema(Schema):
         rschema = self.add_relation_type(ybo.RelationType('identity'))
         rschema.final = False
 
+    etype_name_re = r'[A-Z][A-Za-z0-9]*[a-z]+[A-Za-z0-9]*$'
     def add_entity_type(self, edef):
         edef.name = edef.name.encode()
         edef.name = bw_normalize_etype(edef.name)
-        if not re.match(r'[A-Z][A-Za-z0-9]*[a-z]+[0-9]*$', edef.name):
+        if not re.match(self.etype_name_re, edef.name):
             raise BadSchemaDefinition(
                 '%r is not a valid name for an entity type. It should start '
                 'with an upper cased letter and be followed by at least a '
@@ -645,10 +646,9 @@ def guess_rrqlexpr_mainvars(expression):
 
 def split_expression(rqlstring):
     for expr in rqlstring.split(','):
-        for noparen1 in expr.split('('):
-            for noparen2 in noparen1.split(')'):
-                for word in noparen2.split():
-                    yield word
+        for noparen in expr.split('('):
+            for word in noparen.split():
+                yield word
 
 def normalize_expression(rqlstring):
     """normalize an rql expression to ease schema synchronization (avoid
@@ -1011,9 +1011,6 @@ class RepoEnforcedRQLConstraintMixIn(object):
         else:
             expression = 'S eid %(s)s, O eid %(o)s, ' + self.expression
             args = {'s': eidfrom, 'o': eidto}
-        if 'U' in self.rqlst.defined_vars:
-            expression = 'U eid %(u)s, ' + expression
-            args['u'] = session.user.eid
         rql = 'Any %s WHERE %s' % (','.join(sorted(self.mainvars)), expression)
         if self.distinct_query:
             rql = 'DISTINCT ' + rql
