@@ -247,32 +247,28 @@ def switch_relation_field(sql, table=''):
                                       table + '.eid_from')
     return switchedsql.replace('__eid_from__', table + '.eid_to')
 
-def sort_term_selection(sorts, rqlst, groups):
+def sort_term_selection(sorts, selectedidx, rqlst, groups):
     # XXX beurk
     if isinstance(rqlst, list):
         def append(term):
             rqlst.append(term)
-        selectionidx = set(str(term) for term in rqlst)
     else:
         def append(term):
             rqlst.selection.append(term.copy(rqlst))
-        selectionidx = set(str(term) for term in rqlst.selection)
-
     for sortterm in sorts:
         term = sortterm.term
-        if not isinstance(term, Constant) and not str(term) in selectionidx:
-            selectionidx.add(str(term))
+        if not isinstance(term, Constant) and not str(term) in selectedidx:
+            selectedidx.append(str(term))
             append(term)
             if groups:
                 for vref in term.iget_nodes(VariableRef):
                     if not vref in groups:
                         groups.append(vref)
 
-def fix_selection_and_group(rqlst, needwrap, selectsortterms,
+def fix_selection_and_group(rqlst, selectedidx, needwrap, selectsortterms,
                             sorts, groups, having):
     if selectsortterms and sorts:
-        sort_term_selection(sorts, rqlst, not needwrap and groups)
-    groupvrefs = [vref for term in groups for vref in term.iget_nodes(VariableRef)]
+        sort_term_selection(sorts, selectedidx, rqlst, not needwrap and groups)
     if sorts and groups:
         # when a query is grouped, ensure sort terms are grouped as well
         for sortterm in sorts:
@@ -281,22 +277,19 @@ def fix_selection_and_group(rqlst, needwrap, selectsortterms,
                     (isinstance(term, Function) and
                      get_func_descr(term.name).aggregat)):
                 for vref in term.iget_nodes(VariableRef):
-                    if not vref in groupvrefs:
+                    if not vref in groups:
                         groups.append(vref)
-                        groupvrefs.append(vref)
-    if needwrap and (groups or having):
-        selectedidx = set(vref.name for term in rqlst.selection
-                          for vref in term.get_nodes(VariableRef))
+    if needwrap:
         if groups:
-            for vref in groupvrefs:
-                if vref.name not in selectedidx:
-                    selectedidx.add(vref.name)
+            for vref in groups:
+                if not vref.name in selectedidx:
+                    selectedidx.append(vref.name)
                     rqlst.selection.append(vref)
         if having:
             for term in having:
                 for vref in term.iget_nodes(VariableRef):
-                    if vref.name not in selectedidx:
-                        selectedidx.add(vref.name)
+                    if not vref.name in selectedidx:
+                        selectedidx.append(vref.name)
                         rqlst.selection.append(vref)
 
 def iter_mapped_var_sels(stmt, variable):
@@ -813,16 +806,23 @@ class SQLGenerator(object):
         # treat subqueries
         self._subqueries_sql(select, state)
         # generate sql for this select node
+        selectidx = [str(term) for term in select.selection]
         if needwrap:
             outerselection = origselection[:]
             if sorts and selectsortterms:
+                outerselectidx = [str(term) for term in outerselection]
                 if distinct:
-                    sort_term_selection(sorts, outerselection, groups)
-        fix_selection_and_group(select, needwrap, selectsortterms,
-                                sorts, groups, having)
+                    sort_term_selection(sorts, outerselectidx,
+                                        outerselection, groups)
+            else:
+                outerselectidx = selectidx[:]
+        fix_selection_and_group(select, selectidx, needwrap,
+                                selectsortterms, sorts, groups, having)
         if needwrap:
+            fselectidx = outerselectidx
             fneedwrap = len(outerselection) != len(origselection)
         else:
+            fselectidx = selectidx
             fneedwrap = len(select.selection) != len(origselection)
         if fneedwrap:
             needalias = True
@@ -854,12 +854,8 @@ class SQLGenerator(object):
             # sort
             if sorts:
                 sqlsortterms = []
-                if needwrap:
-                    selectidx = [str(term) for term in outerselection]
-                else:
-                    selectidx = [str(term) for term in select.selection]
                 for sortterm in sorts:
-                    _term = self._sortterm_sql(sortterm, selectidx)
+                    _term = self._sortterm_sql(sortterm, fselectidx)
                     if _term is not None:
                         sqlsortterms.append(_term)
                 if sqlsortterms:
@@ -1251,8 +1247,6 @@ class SQLGenerator(object):
                         return condition
                     self._state.add_outer_join_condition(leftalias, condition)
                 return
-        if leftalias is None:
-            leftalias = leftvar._q_sql.split('.', 1)[0]
         self._state.replace_tables_by_outer_join(
             leftalias, rightalias, outertype, '%s=%s' % (lhssql, rhs.accept(self)))
         return ''
@@ -1372,6 +1366,8 @@ class SQLGenerator(object):
                 operator = ' LIKE '
             else:
                 operator = ' %s ' % operator
+        elif operator == 'REGEXP':
+            return ' %s' % self.dbhelper.sql_regexp_match_expression(rhs.accept(self))
         elif (operator == '=' and isinstance(rhs, Constant)
               and rhs.eval(self._args) is None):
             if lhs is None:
@@ -1422,6 +1418,8 @@ class SQLGenerator(object):
         if constant.type is None:
             return 'NULL'
         value = constant.value
+        if constant.type == 'etype':
+            return value
         if constant.type == 'Int' and  isinstance(constant.parent, SortTerm):
             return value
         if constant.type in ('Date', 'Datetime'):
