@@ -210,12 +210,6 @@ BASIC_WITH_LIMIT = [
 FROM cw_Personne AS _P
 LIMIT 20
 OFFSET 10'''),
-    ("Any P ORDERBY N LIMIT 1 WHERE P is Personne, P travaille S, S eid %(eid)s, P nom N, P nom %(text)s",
-     '''SELECT _P.cw_eid
-FROM cw_Personne AS _P, travaille_relation AS rel_travaille0
-WHERE rel_travaille0.eid_from=_P.cw_eid AND rel_travaille0.eid_to=12345 AND _P.cw_nom=hip hop momo
-ORDER BY _P.cw_nom
-LIMIT 1'''),
     ]
 
 
@@ -954,12 +948,6 @@ WHERE _S.cw_ambiguous_inlined=_A.cw_eid) AS _T0 ON (_X.cw_multisource_inlined_re
      '''SELECT rel_tags0.eid_from, rel_tags0.eid_to, rel_tags1.eid_from
 FROM tags_relation AS rel_tags0 LEFT OUTER JOIN tags_relation AS rel_tags1 ON (rel_tags1.eid_to=123)
 WHERE rel_tags0.eid_from=123'''),
-
-    ('Any CASE, CALIBCFG, CFG '
-     'WHERE CASE eid 1, CFG ecrit_par CASE, CALIBCFG? ecrit_par CASE',
-     '''SELECT _CFG.cw_ecrit_par, _CALIBCFG.cw_eid, _CFG.cw_eid
-FROM cw_Note AS _CFG LEFT OUTER JOIN cw_Note AS _CALIBCFG ON (_CALIBCFG.cw_ecrit_par=_CFG.cw_ecrit_par)
-WHERE _CFG.cw_ecrit_par=1'''),
     ]
 
 VIRTUAL_VARS = [
@@ -1251,7 +1239,7 @@ class PostgresSQLGeneratorTC(RQLGeneratorTC):
 
     def _check(self, rql, sql, varmap=None, args=None):
         if args is None:
-            args = {'text': 'hip hop momo', 'eid': 12345}
+            args = {'text': 'hip hop momo'}
         try:
             union = self._prepare(rql)
             r, nargs, cbs = self.o.generate(union, args,
@@ -1360,6 +1348,18 @@ FROM cw_Personne AS _P''')
                     '''SELECT SUBSTR(_P.cw_nom, 1, 1)
 FROM cw_Personne AS _P''')
 
+    def test_cast(self):
+        self._check("Any CAST(String, P) WHERE P is Personne",
+                    '''SELECT CAST(_P.cw_eid AS text)
+FROM cw_Personne AS _P''')
+
+    def test_regexp(self):
+        self._check("Any X WHERE X login REGEXP '[0-9].*'",
+                    '''SELECT _X.cw_eid
+FROM cw_CWUser AS _X
+WHERE _X.cw_login ~ [0-9].*
+''')
+
     def test_parser_parse(self):
         for t in self._parse(PARSER):
             yield t
@@ -1655,16 +1655,6 @@ FROM owned_by_relation AS rel_owned_by0, cw_CWGroup AS _G LEFT OUTER JOIN in_gro
 WHERE rel_owned_by0.eid_from=_G.cw_eid AND rel_owned_by0.eid_to=1122
 GROUP BY _G.cw_eid''')
 
-    def test_groupby_orderby_insertion_dont_modify_intention(self):
-        self._check('Any YEAR(XECT)*100+MONTH(XECT), COUNT(X),SUM(XCE),AVG(XSCT-XECT) '
-                    'GROUPBY YEAR(XECT),MONTH(XECT) ORDERBY 1 '
-                    'WHERE X creation_date XSCT, X modification_date XECT, '
-                    'X ordernum XCE, X is CWAttribute',
-                    '''SELECT ((CAST(EXTRACT(YEAR from _X.cw_modification_date) AS INTEGER) * 100) + CAST(EXTRACT(MONTH from _X.cw_modification_date) AS INTEGER)), COUNT(_X.cw_eid), SUM(_X.cw_ordernum), AVG((_X.cw_creation_date - _X.cw_modification_date))
-FROM cw_CWAttribute AS _X
-GROUP BY CAST(EXTRACT(YEAR from _X.cw_modification_date) AS INTEGER),CAST(EXTRACT(MONTH from _X.cw_modification_date) AS INTEGER)
-ORDER BY 1'''),
-
 
 class SqlServer2005SQLGeneratorTC(PostgresSQLGeneratorTC):
     backend = 'sqlserver2005'
@@ -1675,6 +1665,9 @@ class SqlServer2005SQLGeneratorTC(PostgresSQLGeneratorTC):
         for t in self._parse(HAS_TEXT_LG_INDEXER):
             yield t
 
+    def test_regexp(self):
+        self.skipTest('regexp-based pattern matching not implemented in sqlserver')
+
     def test_or_having_fake_terms(self):
         self._check('Any X WHERE X is CWUser, X creation_date D HAVING YEAR(D) = "2010" OR D = NULL',
                     '''SELECT _X.cw_eid
@@ -1767,61 +1760,14 @@ SELECT _O.cw_eid, _O.cw_address, _O.cw_alias, _O.cw_modification_date
 FROM cw_EmailAddress AS _O
 WHERE NOT (EXISTS(SELECT 1 FROM use_email_relation AS rel_use_email0 WHERE rel_use_email0.eid_from=1 AND rel_use_email0.eid_to=_O.cw_eid)) AND EXISTS(SELECT 1 FROM use_email_relation AS rel_use_email1 WHERE rel_use_email1.eid_to=_O.cw_eid AND EXISTS(SELECT 1 FROM cw_CWGroup AS _D WHERE rel_use_email1.eid_from=2 AND NOT (EXISTS(SELECT 1 FROM in_group_relation AS rel_in_group2 WHERE rel_in_group2.eid_from=2 AND rel_in_group2.eid_to=_D.cw_eid)) AND _D.cw_name=guests))
 ORDER BY 4 DESC'''),
-
-    ("Any P ORDERBY N LIMIT 1 WHERE P is Personne, P travaille S, S eid %(eid)s, P nom N, P nom %(text)s",
-     '''WITH orderedrows AS (
-SELECT
-_L01
-, ROW_NUMBER() OVER (ORDER BY _L01) AS __RowNumber
-FROM (
-SELECT _P.cw_eid AS _L01 FROM  cw_Personne AS _P, travaille_relation AS rel_travaille0
-WHERE rel_travaille0.eid_from=_P.cw_eid AND rel_travaille0.eid_to=12345 AND _P.cw_nom=hip hop momo
-) AS _SQ1 )
-SELECT
-_L01
-FROM orderedrows WHERE
-__RowNumber <= 1'''),
-
-    ("Any P ORDERBY N LIMIT 1 WHERE P is Personne, P nom N",
-     '''WITH orderedrows AS (
-SELECT
-_L01
-, ROW_NUMBER() OVER (ORDER BY _L01) AS __RowNumber
-FROM (
-SELECT _P.cw_eid AS _L01 FROM  cw_Personne AS _P
-) AS _SQ1 )
-SELECT
-_L01
-FROM orderedrows WHERE
-__RowNumber <= 1
-'''),
-
-    ("Any PN, N, P ORDERBY N LIMIT 1 WHERE P is Personne, P nom N, P prenom PN",
-     '''WITH orderedrows AS (
-SELECT
-_L01, _L02, _L03
-, ROW_NUMBER() OVER (ORDER BY _L02) AS __RowNumber
-FROM (
-SELECT _P.cw_prenom AS _L01, _P.cw_nom AS _L02, _P.cw_eid AS _L03 FROM  cw_Personne AS _P
-) AS _SQ1 )
-SELECT
-_L01, _L02, _L03
-FROM orderedrows WHERE
-__RowNumber <= 1
-'''),
             ]
         for t in self._parse(WITH_LIMIT):# + ADVANCED_WITH_LIMIT_OR_ORDERBY):
             yield t
 
-    def test_groupby_orderby_insertion_dont_modify_intention(self):
-        self._check('Any YEAR(XECT)*100+MONTH(XECT), COUNT(X),SUM(XCE),AVG(XSCT-XECT) '
-                    'GROUPBY YEAR(XECT),MONTH(XECT) ORDERBY 1 '
-                    'WHERE X creation_date XSCT, X modification_date XECT, '
-                    'X ordernum XCE, X is CWAttribute',
-                    '''SELECT ((YEAR(_X.cw_modification_date) * 100) + MONTH(_X.cw_modification_date)), COUNT(_X.cw_eid), SUM(_X.cw_ordernum), AVG((_X.cw_creation_date - _X.cw_modification_date))
-FROM cw_CWAttribute AS _X
-GROUP BY YEAR(_X.cw_modification_date),MONTH(_X.cw_modification_date)
-ORDER BY 1'''),
+    def test_cast(self):
+        self._check("Any CAST(String, P) WHERE P is Personne",
+                    '''SELECT CAST(_P.cw_eid AS nvarchar(max))
+FROM cw_Personne AS _P''')
 
 
 class SqliteSQLGeneratorTC(PostgresSQLGeneratorTC):
@@ -1835,6 +1781,14 @@ class SqliteSQLGeneratorTC(PostgresSQLGeneratorTC):
                     '''SELECT MONTH(_P.cw_creation_date)
 FROM cw_Personne AS _P''')
 
+    def test_regexp(self):
+        self._check("Any X WHERE X login REGEXP '[0-9].*'",
+                    '''SELECT _X.cw_eid
+FROM cw_CWUser AS _X
+WHERE _X.cw_login REGEXP [0-9].*
+''')
+
+
     def test_union(self):
         for t in self._parse((
             ('(Any N ORDERBY 1 WHERE X name N, X is State)'
@@ -1953,16 +1907,6 @@ WHERE appears0.word_id IN (SELECT word_id FROM word WHERE word in ('toto', 'tata
 FROM cw_CWUser AS _X
 WHERE ((YEAR(_X.cw_creation_date)=2010) OR (_X.cw_creation_date IS NULL))''')
 
-    def test_groupby_orderby_insertion_dont_modify_intention(self):
-        self._check('Any YEAR(XECT)*100+MONTH(XECT), COUNT(X),SUM(XCE),AVG(XSCT-XECT) '
-                    'GROUPBY YEAR(XECT),MONTH(XECT) ORDERBY 1 '
-                    'WHERE X creation_date XSCT, X modification_date XECT, '
-                    'X ordernum XCE, X is CWAttribute',
-                    '''SELECT ((YEAR(_X.cw_modification_date) * 100) + MONTH(_X.cw_modification_date)), COUNT(_X.cw_eid), SUM(_X.cw_ordernum), AVG((_X.cw_creation_date - _X.cw_modification_date))
-FROM cw_CWAttribute AS _X
-GROUP BY YEAR(_X.cw_modification_date),MONTH(_X.cw_modification_date)
-ORDER BY 1'''),
-
 
 
 class MySQLGenerator(PostgresSQLGeneratorTC):
@@ -1985,6 +1929,18 @@ class MySQLGenerator(PostgresSQLGeneratorTC):
                     '''SELECT EXTRACT(MONTH from _P.cw_creation_date)
 FROM cw_Personne AS _P''')
 
+    def test_cast(self):
+        self._check("Any CAST(String, P) WHERE P is Personne",
+                    '''SELECT CAST(_P.cw_eid AS mediumtext)
+FROM cw_Personne AS _P''')
+
+    def test_regexp(self):
+        self._check("Any X WHERE X login REGEXP '[0-9].*'",
+                    '''SELECT _X.cw_eid
+FROM cw_CWUser AS _X
+WHERE _X.cw_login REGEXP [0-9].*
+''')
+
     def test_from_clause_needed(self):
         queries = [("Any 1 WHERE EXISTS(T is CWGroup, T name 'managers')",
                     '''SELECT 1
@@ -2059,16 +2015,6 @@ WHERE ((EXTRACT(YEAR from _X.cw_creation_date)=2010) OR (_X.cw_creation_date IS
 FROM (SELECT 1) AS _T
 WHERE NOT (EXISTS(SELECT 1 FROM in_group_relation AS rel_in_group0))''')
 
-    def test_groupby_orderby_insertion_dont_modify_intention(self):
-        self._check('Any YEAR(XECT)*100+MONTH(XECT), COUNT(X),SUM(XCE),AVG(XSCT-XECT) '
-                    'GROUPBY YEAR(XECT),MONTH(XECT) ORDERBY 1 '
-                    'WHERE X creation_date XSCT, X modification_date XECT, '
-                    'X ordernum XCE, X is CWAttribute',
-                    '''SELECT ((EXTRACT(YEAR from _X.cw_modification_date) * 100) + EXTRACT(MONTH from _X.cw_modification_date)), COUNT(_X.cw_eid), SUM(_X.cw_ordernum), AVG((_X.cw_creation_date - _X.cw_modification_date))
-FROM cw_CWAttribute AS _X
-GROUP BY EXTRACT(YEAR from _X.cw_modification_date),EXTRACT(MONTH from _X.cw_modification_date)
-ORDER BY 1'''),
-
 
 class removeUnsusedSolutionsTC(TestCase):
     def test_invariant_not_varying(self):
