@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
@@ -22,11 +21,9 @@ from cubicweb import ValidationError
 from cubicweb.devtools.testlib import CubicWebTC
 from cubicweb.transaction import *
 
-from cubicweb.server.sources.native import UndoException
-
-
 class UndoableTransactionTC(CubicWebTC):
 
+        
     def setup_database(self):
         req = self.request()
         self.session.undo_actions = set('CUDAR')
@@ -153,8 +150,8 @@ class UndoableTransactionTC(CubicWebTC):
         txuuid = self.commit()
         actions = self.cnx.transaction_info(txuuid).actions_list()
         self.assertEqual(len(actions), 1)
-        toto.clear_all_caches()
-        e.clear_all_caches()
+        toto.cw_clear_all_caches()
+        e.cw_clear_all_caches()
         errors = self.cnx.undo_transaction(txuuid)
         undotxuuid = self.commit()
         self.assertEqual(undotxuuid, None) # undo not undoable
@@ -195,7 +192,7 @@ class UndoableTransactionTC(CubicWebTC):
         self.commit()
         errors = self.cnx.undo_transaction(txuuid)
         self.commit()
-        p.clear_all_caches()
+        p.cw_clear_all_caches()
         self.assertEqual(p.fiche[0].eid, c2.eid)
         self.assertEqual(len(errors), 1)
         self.assertEqual(errors[0],
@@ -235,7 +232,7 @@ class UndoableTransactionTC(CubicWebTC):
         self.failIf(self.execute('Any X WHERE X eid %(x)s', {'x': c.eid}))
         self.failIf(self.execute('Any X WHERE X eid %(x)s', {'x': p.eid}))
         self.failIf(self.execute('Any X,Y WHERE X fiche Y'))
-        self.session.set_pool()
+        self.session.set_cnxset()
         for eid in (p.eid, c.eid):
             self.failIf(session.system_sql(
                 'SELECT * FROM entities WHERE eid=%s' % eid).fetchall())
@@ -288,15 +285,6 @@ class UndoableTransactionTC(CubicWebTC):
 
     # test implicit 'replacement' of an inlined relation
 
-
-class UndoExceptionInUnicode(CubicWebTC):
-
-    # problem occurs in string manipulation for python < 2.6
-    def test___unicode__method(self):
-        u = UndoException(u"voilà")
-        self.assertIsInstance(unicode(u), unicode)
-
-
 if __name__ == '__main__':
     from logilab.common.testlib import unittest_main
     unittest_main()
