@@ -102,7 +102,7 @@ class LogoutController(Controller):
         msg = self._cw._('you have been logged out')
         # force base_url so on dual http/https configuration, we generate an url
         # on the http version of the site
-        return self._cw.build_url('view', vid='index', __message=msg,
+        return self._cw.build_url('view', vid='loggedout',
                                   base_url=self._cw.vreg.config['base-url'])
 
 
@@ -603,14 +603,16 @@ class UndoController(Controller):
         errors = self._cw.cnx.undo_transaction(txuuid)
         if not errors:
             self.redirect()
-        raise ValidationError(None, {None: '\n'.join(errors)})
+        return self._cw._('some errors occurred:') + self._cw.view(
+            'pyvalist', pyvalue=errors)
 
-    def redirect(self, msg=None):
+    def redirect(self):
         req = self._cw
-        msg = msg or req._("transaction undone")
         breadcrumbs = req.session.data.get('breadcrumbs', None)
         if breadcrumbs is not None and len(breadcrumbs) > 1:
-            url = req.rebuild_url(breadcrumbs[-2], __message=msg)
+            url = req.rebuild_url(breadcrumbs[-2],
+                                  __message=req._('transaction undoed'))
         else:
-            url = req.build_url(__message=msg)
+            url = req.build_url(__message=req._('transaction undoed'))
         raise Redirect(url)
+
