@@ -27,7 +27,7 @@ from itertools import repeat, chain
 from cubicweb.selectors import is_instance, score_entity, match_user_groups
 from cubicweb.view import EntityView, StartupView
 from cubicweb.schema import META_RTYPES, VIRTUAL_RTYPES, display_name
-from cubicweb.web import uicfg, formwidgets as wdgs
+from cubicweb.web import uicfg
 from cubicweb.web.views import tabs, actions
 
 
@@ -35,12 +35,6 @@ _abaa = uicfg.actionbox_appearsin_addmenu
 _abaa.tag_object_of(('CWSourceSchemaConfig', 'cw_schema', '*'), False)
 _abaa.tag_object_of(('CWSourceSchemaConfig', 'cw_for_source', '*'), False)
 
-_afs = uicfg.autoform_section
-_afs.tag_attribute(('CWSource', 'synchronizing'), 'main', 'hidden')
-_afs.tag_object_of(('*', 'cw_for_source', 'CWSource'), 'main', 'hidden')
-_affk = uicfg.autoform_field_kwargs
-_affk.tag_attribute(('CWSource', 'parser'), {'widget': wdgs.TextInput})
-
 # source primary views #########################################################
 
 _pvs = uicfg.primaryview_section
@@ -229,7 +223,8 @@ class ManageSourcesAction(actions.ManagersAction):
 
 class CWSourceManagementView(StartupView):
     __regid__ = 'cw.source-management'
-    rql = ('Any S, ST, SN ORDERBY SN WHERE S is CWSource, S name SN, S type ST')
+    rql = ('Any S, ST, SP, SD, SN ORDERBY SN WHERE S is CWSource, S name SN, S type ST, '
+           'S latest_retrieval SD, S parser SP')
     title = _('data sources management')
 
     def call(self, **kwargs):
@@ -240,4 +235,4 @@ class CWSourceManagementView(StartupView):
                 self._cw.build_url('add/%s' % eschema),
                 self._cw._('add a CWSource')))
             self.w(u'<div class="clear"></div>')
-        self.wview('table', self._cw.execute(self.rql), displaycols=range(2))
+        self.wview('table', self._cw.execute(self.rql), displaycols=range(4))
